# add_quiz_question.py
from typing import Dict, List, Optional
import click
from tomli_w import dump
from quecital.quiz import main_loop
from quecital.multiline_text_editor import get_multiline_edit


def main(quecital_toml_path, quecital_data):
    """
    Add a new question to the quiz.

    Args:
        quecital_toml_path (str): Path to the quiz TOML file.
        quecital_data (dict): Dictionary containing quiz data.
    """
    topic_question_assets = get_questions_for_topic(quecital_data)
    new_question_asset: dict = form_toml_entry()
    topic_question_assets.append(new_question_asset)
    with open(quecital_toml_path, "wb") as f:
        dump(quecital_data, f)


def get_questions_for_topic(quecital_data) -> List[Dict]:
    """
    Return the list of questions for the selected topic.

    Args:
        quecital_data (dict): Dictionary containing quiz data.

    Returns:
        List[Dict]: A list of dictionaries where each dictionary represents
            a question.
    """
    topic_label = main_loop(
        question="Which topic do you want to add a question to",
        alternatives=quecital_data.keys(),
    )[0]
    return quecital_data[topic_label]["questions"]


#  TODO handle the key error if ["questions"] not in quecital_data[topic_label]


def form_toml_entry() -> Dict[str, str | List | Optional[str]]:
    """
    Gather information to form a new quiz entry.

    Returns:
        Dict: A dictionary representing a new quiz question.
    """
    question: str = rigid_receiver("question")

    answers: List[str] = flexible_receiver("answers")

    alternatives: List[str] = flexible_receiver("alternatives")

    hint: Optional[str] = rigid_receiver("hint")

    explanation: Optional[str] = rigid_receiver("explanation")

    return {
        "question": question,
        "answers": answers,
        "alternatives": alternatives,
        "hint": hint,
        "explanation": explanation,
    }


def rigid_receiver(config: str) -> str:
    """
    Receive input for quiz assets that don't support multiple lines.

    Args:
        config (str): Key representing the type of input.

    Returns:
        str: The received input.
    """

    configs = {
        "question": {
            "garnish": "Enter a question prompt",
        },
        "hint": {
            "garnish": "Offer a hint? - enter to skip",
        },
        "explanation": {
            "garnish": "Offer an explanation? - enter to skip",
        },
    }

    if config in configs:
        user_input: str | List = get_multiline_edit(configs[config]["garnish"])
        if isinstance(user_input, str):
            return str(user_input)
        raise TypeError("User input is not a string")
    raise KeyError("config not in configs")


def flexible_receiver(config) -> List:
    """
    Receive input for quiz assets that support multiple lines.

    Args:
        config: Key representing the type of input.

    Returns:
        List: List of received inputs.
    """
    configs: Dict[str, Dict[str, str | bool]] = {
        "answers": {
            "garnish": "Enter a correct answer",
            "helpings": "Add a second required answer for this question?",
            "default_boolean": False,
        },
        "alternatives": {
            "garnish": "Enter an alternative, i.e., an incorrect answer",
            "helpings": "Add another alternative?",
            "default_boolean": True,
        },
    }
    assets: List = []
    if config in configs:
        while True:
            user_input: str | List = get_multiline_edit(
                # str() makes the static typer chill
                str(configs[config]["garnish"])
            )
            match user_input:
                case str(user_input):
                    assets.append(user_input)
                case list(user_input):
                    assets.extend(user_input)
                case _:
                    raise TypeError("Unsupported datatype entered for answer.")

            another_helping = click.confirm(
                # str() and bool() needed to soothe the static typer
                text=str(configs[config]["helpings"]),
                default=bool(configs[config]["default_boolean"]),
            )
            if not another_helping:
                break
    return assets



def answer() -> List:
    """Support the user submitting a prompt with
    multiple correct answers as one correct answer per line or
    one correct multiline answer.

    Multiple sequential prompts can be submitted.

    The functionality for one correct answer per line is available
    via ptk keybinding in multiline_text_editor.get_multiline_edit()
    """
    garnish = "Enter a correct answer: "
    answers = []
    while True:
        user_input = get_multiline_edit(garnish)
        match user_input:
            case str(user_input):
                answers.append(user_input)
            case list(user_input):
                answers.extend(user_input)
            case _:
                raise TypeError("Unsupported datatype entered for answer.")

        add_another_of_the_same = click.confirm(
            "Add a second required answer for this question?", default=False
        )
        if not add_another_of_the_same:
            break
    return answers



def alternative() -> List:
    garnish = "Enter an alternative, i.e., an incorrect answer"
    alternatives = []
    while True:
        user_input = get_multiline_edit(garnish)
        match user_input:
            case str(user_input):
                alternatives.append(user_input)
            case list(user_input):
                alternatives.extend(user_input)
            case _:
                raise TypeError("Unsupported datatype entered for answer.")

        add_another_of_the_same = click.confirm(
            "Add another alternative?", default=True
        )
        if not add_another_of_the_same:
            break
    return alternatives


# recital.py
"""
This module provides functions for conducting random recitals
and displaying recital details.
"""
import click
from pathlib import Path
import tomllib
import random
from quecital.multiline_recital_editor import get_multiline_edit


def main(data):
    """
    Conduct a random recital and display details including the prompt, attempt, and correct answer.

    Args:
        data (dict): A dictionary containing recital data.

    Returns:
        None
    """
    assets = preprocess(data)
    attempt = get_attempt(assets)
    display = f"\
        \nThe prompt was:\
        \n{assets['question']}\n\
        \nThe attempt was:\
        \n{attempt}\n\
        \nThe correct answer is:\
        \n{assets['answers']}\n"
    if "explanation" in assets and assets["explanation"]:
        display += f"\n\nExplanation: {assets['explanation']}"
    click.echo(display)


def compose_prompt(assets, show_hint=False):
    """
    Compose the prompt for a recital, optionally including a hint.

    Args:
        assets (dict): A dictionary containing recital assets.
        show_hint (bool, optional): A flag indicating whether to include the hint. Defaults to False.

    Returns:
        str: The composed prompt.
    """
    hint = "Hint not provided"
    if "hint" in assets and assets["hint"] and show_hint is False:
        hint = "<DELETE FOR HINT>"
    if "hint" in assets and assets["hint"] and show_hint is True:
        hint = assets["hint"]
    prompt = f'The prompt is:\n\n{assets["question"]}\n\n{hint}'
    return prompt


def get_attempt(assets):
    """
    Get the user's attempt for a recital, handling the option to
    view the hint.

    Args:
        assets (dict): A dictionary containing recital assets.

    Returns:
        str: The user's attempt.
    """
    prompt = compose_prompt(assets)
    attempt = get_multiline_edit(prompt)
    if attempt == "show_hint":
        prompt = compose_prompt(assets, show_hint=True)
        attempt = get_multiline_edit(prompt, hint_shown=True)

    return attempt


def preprocess(quecital_data):
    """
    Preprocess quecital data and prompt the user to choose a topic for a random recital.

    Args:
        quecital_data (dict): A dictionary containing quecital data.

    Returns:
        dict: Recital assets for the chosen topic.
    """
    topics = list(quecital_data.keys())
    prompt = "\nFrom which topic will you perform a random recital\n\n"
    for index, topic in enumerate(topics, start=1):
        prompt += f"\n{index}: {topic}"
    prompt += "\n\nEnter one of the following ->"
    valid_choices = list(map(str, range(1, len(topics) + 1)))
    selected_index = click.prompt(prompt, type=click.Choice(valid_choices))
    topic = topics[int(selected_index) - 1]
    return random.choice(quecital_data[topic]["recitals"])


if __name__ == "__main__":
    quecital_path = Path("quecital.toml")
    data = tomllib.loads(quecital_path.read_text(encoding="utf-8"))
    main(data)



# multiline_recital_editor.py
"""This module provides a function for capturing multiline input from the user
using an interactive editor.
"""
import click


def get_multiline_edit(prompt, hint_shown=False):
    """
    Capture multiline input from the user using an interactive editor.

    The function opens an interactive editor (specified by the user or the
    default) for multiline input. The user can edit the content and save
    the changes.

    Args:
        prompt (str): The prompt to display to the user.
        hint_shown (bool, optional): A flag indicating whether a hint is
        shown to the user. Defaults to False.

    Returns:
        str: The user's multiline input or 'show_hint' if the user wants to
        see the hint.
             Returns an empty string if no changes were made.
    """
    # Custom marker to separate prompt text from user's input
    custom_marker = "# ---- Your Input Below ----\n"
    # Flags for hint behavior
    hide_hint = '<DELETE FOR HINT>'
    no_hint = 'Hint not provided'
    # the message displayed with the editor
    # the hint flags are included in the prompt decided by recital.py
    default_content = f"{prompt}\n\n{custom_marker}"
    # the interactive editor, TODO improve parameters
    edited_content = click.edit(
        text=default_content,
        editor=None,
    )

    # Now we start managing the hint behavior. Perform membership tests on
    # the interactive editor's output. Look for the hint flags. Hints are not
    # required -> "Hint not provided." Hints can be available to the user but
    # not viewed -> "<DELETE FOR HINT>". If the hint is requested, (TODO:
    # don't lose any work done before requesting the hint),

    # recital.py flips the hint_shown flag on this get_multiline_edit()
    # when the interactive editor senses a change and hint_shown flag is True,
    # the program finishes (return user)
    if edited_content and hint_shown is True:
        # Find the position of the custom marker and extract user's input
        marker_position = edited_content.find(custom_marker)
        user_input = edited_content[marker_position + len(custom_marker):].strip()
        return user_input

    # check if the user wants to see the hint
    # tell recital.get_attempt to attach the hint to the prompt and send back
    # "if (this is true) and (this is of value that)"
    if edited_content and hint_shown is False:
        if hide_hint not in edited_content and no_hint not in edited_content:
            return 'show_hint'
    # an attempt without viewing the hint
    if edited_content and hint_shown is False:
        if hide_hint in edited_content or no_hint in edited_content:
            # Find the position of the custom marker and extract user's input
            marker_position = edited_content.find(custom_marker)
            user_input = edited_content[marker_position + len(custom_marker):].strip()

            return user_input

    else:
        click.echo("No changes were made.")
        return ""


if __name__ == "__main__":
    # Example of using get_multiline_edit()
    code_input = get_multiline_edit("Enter your code:")

    if code_input:
        click.echo("\nYou entered:")
        for line in code_input:
            click.echo(line)



from typing import Union, List
import click
from prompt_toolkit import prompt
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.history import InMemoryHistory
from prompt_toolkit.styles import Style


# Constants
INPUT_BELOW_MARKER = "# ---- Your Input Below ----\n"
MULTI_FLAG = "MULTI_FLAG"


def get_default_content(garnish):
    header = f"{garnish}\n\n{INPUT_BELOW_MARKER}"
    return header


def extract_user_input(user_input: str, multi_flag=False) -> str | List[str]:
    below_marker_position = user_input.find(INPUT_BELOW_MARKER)

    # user_input is one multiline asset from one window
    # return a string
    if multi_flag is False and below_marker_position != -1:
        return user_input[below_marker_position + len(INPUT_BELOW_MARKER) :].strip()

    # user_input are multiple single line assets from one window
    # return a list of strings
    if multi_flag is True and below_marker_position != -1:
        user_input = user_input[
            below_marker_position + len(INPUT_BELOW_MARKER) :
        ].strip()
        return user_input.split("\n")

    click.echo("No changes were made.")
    return ""


def bottom_toolbar():
    # use a list of style/text tuples
    return [
        ("class:bottom-toolbar-translucent", ""),
        ("class:bottom-toolbar-opaque", "\n Ctrl+C to quit, alt+enter to proceed "),
        ("class:bottom-toolbar-opaque", "F2 to save one as one asset entry per line"),
    ]


def inject_multi_flag(prompt_output):
    return MULTI_FLAG + prompt_output


def get_multiline_edit(garnish: str) -> str | List[str]:
    default_content = get_default_content(garnish)

    history = InMemoryHistory()

    style = Style.from_dict(
        {
            "bottom-toolbar-translucent": "#ffffff bg:#444400",
            "bottom-toolbar-opaque": "#ffffff bg:#333333",
        }
    )
    kb = KeyBindings()

    @kb.add("f2")
    def _(event):
        event.app.exit(inject_multi_flag(event.app.current_buffer.text))

    try:
        user_input: str = prompt(
            default=default_content,
            multiline=True,
            history=history,
            bottom_toolbar=bottom_toolbar,
            style=style,
            key_bindings=kb,
        )
    except (EOFError, KeyboardInterrupt):
        user_input = ""

    if MULTI_FLAG in user_input:
        return extract_user_input(user_input, multi_flag=True)
    return extract_user_input(user_input)


if __name__ == "__main__":
    code_input = get_multiline_edit("Enter your code:")

    if code_input:
        click.echo("\nYou entered:")
        click.echo(code_input)
    else:
        click.echo("\nNo input received.")
